%
% matlab script to process the file with quanrtizer output
%

% Input data:
load out.sig+Q;  outq=out;
load out.sig+dev+Q;
period = 80e-9;
Vref = 1.3;
fsig = 1/3.2768e-04;
N = length(out);
fstart= 4e3;
fstop= 1e5;
Npsd = N;

% Transform 0,1 wavefroms to -2Vref, 2Vref (consider differential output)
outq = 4*Vref*(outq-0.5);
out  = 4*Vref*(out -0.5);

[Stdq,Ftd] = psd(outq,  Npsd, 1/period, Npsd );
[Std, Ftd] = psd(out,  Npsd, 1/period, Npsd );
% Scale to preserve total energy in (0, fs/2)
Stdq = (2*period) * Stdq;
Std  = (2*period) * Std;

% Integrate the bins around fsig to calclulate signal power 
% This is the same from Stdq, and is also known in advance = 4*Vo^2/2, 
% where Vo is the input sinusoid ampitude (here Vo=0.65);
Nsig = round(fsig*period*N)+1;
Psig = sum(Std(Nsig-3:Nsig+3)) * 1/(period*Npsd);
Std(Nsig-3:Nsig+3)=0;
Stdq(Nsig-3:Nsig+3)=0;

Psig = 4*0.65^2/2;

% Input referred noise from simulation with SpectreRF
load devnoise.in;
devF=devnoise(:,1);
devN=devnoise(:,2);

NF = length(devF);
Kstart = round(NF * log(fstart/devF(1)) / log(devF(NF)/devF(1)) );
Kstop  = round(NF * log(fstop /devF(1)) / log(devF(NF)/devF(1)) );
points = Kstop - Kstart;
snrn = zeros(points,1);
snrq = zeros(points,1);
snrc = zeros(points,1);

for k=Kstart+1:Kstop,
k
fco = devF(k)*1.000001;
i=2;
devNpow=0;
while (devF(i) < fco)
  devNpow = devNpow + 0.5*(abs(devN(i-1))^2 + abs(devN(i))^2) * (devF(i) - devF(i-1));
  i=i+1;
end;
snrc(k-Kstart) = 10*log10(Psig/devNpow);
Nco = round(fco*period*Npsd);
snrn(k-Kstart) = 10*log10(4*Psig / (sum(Std (1:Nco))/(period*Npsd)) );
snrq(k-Kstart) = 10*log10(4*Psig / (sum(Stdq(1:Nco))/(period*Npsd)) );
end;

figure(51);
FF = devF(Kstart+1:Kstop);
semilogx( FF, snrc, 'rs-', FF, snrn, 'bx-', FF, snrq, 'go-'); 
xlabel('Cut-off Frequency (Hz)');
ylabel('SNR (dB)');
axis([4e3 1e5 70 140]);
print -depsc 'snrplot.eps'

